"""
@Author: kang.yang
@Date: 2023/5/12 20:49
"""
import os
import time

import allure
from playwright.sync_api import sync_playwright, expect, Locator

from kuto.testdata import get_int
from kuto.utils.exceptions import ScreenFailException
from kuto.utils.log import logger


class PlayWrightDriver:

    def __init__(self, browserName: str, headless: bool = False, state: dict = None):
        self.browserName = browserName
        self.headless = headless

        self.playwright = sync_playwright().start()
        if browserName == 'firefox':
            self.browser = self.playwright.firefox.launch(headless=headless)
        elif browserName == 'webkit':
            self.browser = self.playwright.webkit.launch(headless=headless)
        else:
            self.browser = self.playwright.chromium.launch(headless=headless)
        if state:
            self.context = self.browser.new_context(storage_state=state, no_viewport=True)
        else:
            self.context = self.browser.new_context(no_viewport=True)
        self.page = self.context.new_page()

    def switch_tab(self, locator: Locator):
        with self.page.expect_popup() as popup_info:
            locator.click()
        self.page = popup_info.value

    def visit(self, url):
        logger.info(f"visit: {url}")
        self.page.goto(url)

    def storage_state(self, path=None):
        logger.info("save state")
        if not path:
            raise ValueError("path can not be None.")
        self.context.storage_state(path=path)

    def screenshot(self, file_name=None, with_time=True):
        """
        截图并保存到预定路径
        @param with_time: 是否带时间戳
        @param file_name: foo.png or fool
        @return:
        """
        if not file_name:
            raise ValueError("file_name should not be None.")

        try:
            # 把文件名处理成test.png的样式
            if "." in file_name:
                file_name = file_name.split(r".")[0]
            # 截图并保存到当前目录的images文件夹中
            relative_path = "screenshots"
            if os.path.exists(relative_path) is False:
                os.mkdir(relative_path)
            if with_time:
                time_str = time.strftime(f"%Y-%m-%d_%H_%M_%S_{get_int(min_size=1, max_size=1000)}")
                file_name = f"{time_str}_{file_name}.png"
            else:
                file_name = f'{file_name}.png'
            file_path = os.path.join(relative_path, file_name)
            logger.info(f"save to: {os.path.join(relative_path, file_name)}")
            self.page.screenshot(path=file_path, full_page=True)
            # 上传allure报告
            allure.attach.file(
                file_path,
                attachment_type=allure.attachment_type.PNG,
                name=f"{file_name}",
            )
            return file_path
        except Exception as e:
            raise ScreenFailException(f"screen failed: \n{str(e)}")

    def close(self):
        logger.info("close browser")
        self.page.close()
        self.context.close()
        self.browser.close()
        self.playwright.stop()

    def assert_title(self, title: str):
        logger.info("assert title")
        expect(self.page).to_have_title(title)

    def assert_url(self, url: str):
        logger.info("assert url")
        expect(self.page).to_have_url(url)



"""
@Author: kang.yang
@Date: 2023/5/16 14:37
"""
import kuto
from kuto import WebElem


class IndexPage(kuto.Page):
    """首页"""
    login = WebElem(text='登录/注册')
    enterprise = WebElem(text='查专利')


class LoginPage(kuto.Page):
    """登录页"""
    pwdLogin = WebElem(text='帐号密码登录')
    userInput = WebElem(placeholder='请输入手机号码')
    pwdInput = WebElem(placeholder='请输入密码')
    licenseBtn = WebElem(css="span.el-checkbox__inner", index=1)
    loginBtn = WebElem(text='立即登录')


class TestLogin(kuto.TestCase):
    """登录测试"""

    def start(self):
        self.index_page = IndexPage(self.d)
        self.login_page = LoginPage(self.d)

    def test_1(self):
        """账号密码登录"""
        self.open()
        self.index_page.login.click()
        self.login_page.pwdLogin.click()
        self.login_page.userInput.input('xxx')
        self.login_page.pwdInput.input('xxx')
        self.login_page.licenseBtn.click()
        self.login_page.loginBtn.click()
        self.d.screenshot("登录成功")


if __name__ == '__main__':
    # with open("state.json", "r") as f:
    #     state = json.loads(f.read())

    kuto.main(browser="chrome", host="https://www.qizhidao.com")



import inspect
import typing

from uiautomator2 import UiObject
from uiautomator2.xpath import XPathSelector

from kuto.core.android.driver import AndroidDriver
from kuto.utils.exceptions import NoSuchElementException
from kuto.utils.log import logger
from kuto.utils.config import config


class AdrElem(object):
    """
    安卓元素定义
    """

    def __init__(self,
                 driver: AndroidDriver = None,
                 resourceId: str = None,
                 className: str = None,
                 text: str = None,
                 xpath: str = None,
                 index: int = 0):
        """
        @param driver: 安卓驱动，必填
        @param resourceId: resourceId定位
        @param className: className定位
        @param text: text定位
        @param xpath: xpath定位
        @param index: 定位出多个元素时，指定索引
        """
        self._driver = driver

        self._kwargs = {}
        if resourceId is not None:
            self._kwargs["resourceId"] = resourceId
        if className is not None:
            self._kwargs["className"] = className
        if text is not None:
            self._kwargs["text"] = text

        self._xpath = xpath
        self._index = index

    def __get__(self, instance, owner):
        if instance is None:
            return None

        self._driver = instance.driver
        return self

    # def error_handler(self):
    #     """异常处理，暂时只支持resourceId和text"""
    #     errors = config.get_app('errors')
    #     if errors:
    #         logger.info(f'deal errors: {errors}')
    #         for error in errors:
    #             self._driver.d(**error).click_exists()

    # def find_element(self, retry=3, timeout=3):
    #     """
    #     为了留出异常处理的逻辑，所以加了一个find_element的方法，不然可以合并到get_element方法
    #     @param retry: 重试次数
    #     @param timeout: 每次查找时间
    #     @return:
    #     """
    #     if self._xpath is not None:
    #         logger.info(f'find: xpath={self._xpath}')
    #     else:
    #         logger.info(f'find: {self._kwargs}[{self._index}]')
    #     _element = self._driver.d.xpath(self._xpath) if \
    #         self._xpath is not None else self._driver.d(**self._kwargs)[self._index]
    #     while not _element.wait(timeout=timeout):
    #         if retry > 0:
    #             retry -= 1
    #             # self.error_handler()
    #             logger.warning(f'retry： {self._kwargs},{self._index}')
    #         else:
    #             frame = inspect.currentframe().f_back
    #             caller = inspect.getframeinfo(frame)
    #             logger.warning(f'【{caller.function}:{caller.lineno}】find not {self._kwargs}')
    #             return None
    #     return _element

    def get_element(self, timeout=5):
        """
        增加截图的方法
        @param timeout: 每次查找时间
        @return:
        """
        if self._xpath is not None:
            logger.info(f'find: xpath={self._xpath}')
        else:
            logger.info(f'find: {self._kwargs}[{self._index}]')
        _element = self._driver.d.xpath(self._xpath) if \
            self._xpath is not None else self._driver.d(**self._kwargs)[self._index]

        if _element.wait(timeout=timeout):
            return _element
        else:
            logger.info("loc fail")
            self._driver.screenshot("loc_fail")
            raise NoSuchElementException(f"[elem {self._kwargs} loc fail]")

    @property
    def info(self):
        logger.info(f"get info")
        return self.get_element(timeout=1).info

    @property
    def text(self):
        logger.info(f"get text")
        return self.get_element().info.get("text")

    @property
    def bounds(self):
        logger.info(f"get bounds")
        return self.get_element().info.get("bounds")

    def exists(self, timeout=3):
        logger.info(f"if exists")
        try:
            self.get_element(timeout=timeout)
        except:
            return False
        else:
            return True

    @staticmethod
    def _adapt_center(e: typing.Union[UiObject, XPathSelector], offset=(0.5, 0.5)):
        if isinstance(e, UiObject):
            return e.center(offset=offset)
        else:
            return e.offset(offset[0], offset[1])

    def click(self):
        logger.info("click")
        element = self.get_element()
        # 这种方式经常点击不成功，感觉是页面刷新有影响
        # element.click()
        x, y = self._adapt_center(element)
        # if config.get_app('double_check'):
        #     logger.info('检查点击结果.')
        #     info_before = element.info
        #     self._driver.d.click(x, y)
        #     # 判断点击前后元素信息是否相同，如果相同就再点击一次
        #     if element.exists:
        #         if element.info == info_before:
        #             logger.debug('点击失败，再点一次')
        #             self.error_handler()
        #             self._driver.d.click(x, y)
        # else:
        #     self._driver.d.click(x, y)
        self._driver.d.click(x, y)

    def click_exists(self, timeout=3):
        logger.info(f"click if exists")
        if self.exists(timeout=timeout):
            self.click()

    def input(self, text):
        logger.info(f"input: {text}")
        self.get_element().set_text(text)

    def input_pwd(self, text):
        logger.info(f'input password: {text}')
        self.get_element().click()
        self._driver.d(focused=True).set_text(text)

    def clear(self):
        logger.info("clear text")
        self.get_element().clear_text()

    # def drag_to(self, *args, **kwargs):
    #     logger.info(f"drag to")
    #     self.get_element().drag_to(*args, **kwargs)
    #
    # def swipe_left(self):
    #     logger.info(f"swipe left")
    #     self.get_element().swipe("left")
    #
    # def swipe_right(self):
    #     logger.info(f"swipe right")
    #     self.get_element().swipe("right")
    #
    # def swipe_up(self):
    #     logger.info(f"swipe up")
    #     self.get_element().swipe("up")
    #
    # def swipe_down(self):
    #     logger.info(f"swipe down")
    #     self.get_element().swipe("down")


if __name__ == '__main__':
    pass





import re
import sys
import time
import uuid
import random
import hashlib
import datetime

from .data import (
    en_first_names_male,
    en_first_names_female,
    en_last_names,
    zh_names_male,
    zh_names_female,
    zh_last_name,
    words_str,
    mobile,
    unicom,
    telecom,
)


def first_name(gender: str = "", language: str = "en") -> str:
    """
    get first name
    :param gender:
    :param language:
    :return:
    """
    genders = ["", "m", "f", "male", "female"]
    if gender not in genders:
        raise ValueError("Unsupported gender, try [m, f, male, female] instead")

    if language == "en":
        if gender == "":
            name = random.choice(en_first_names_female + en_first_names_male)
        elif gender in ["m", "male"]:
            name = random.choice(en_first_names_male)
        else:
            name = random.choice(en_first_names_female)

        return name.capitalize()

    elif language == "zh":
        if gender == "":
            name = random.choice(zh_names_female + zh_names_male)
        elif gender == "m":
            name = random.choice(zh_names_male)
        else:
            name = random.choice(zh_names_female)

        return name


def last_name(language: str = "en") -> str:
    """
    get last name
    :return:
    """
    if language == "en":
        name = random.choice(en_last_names)
        return name.capitalize()
    elif language == "zh":
        name = random.choice(zh_last_name)
        return name
    else:
        raise ValueError(f"{language} language is not supported")


def username(name: str = "", language: str = "en") -> str:
    """
    this is a very basic username generator
    """
    if language == "en":
        if not name:
            name = first_name() if yes() else last_name()
        name = re.sub(r"['-]", "", name)
        return name
    elif language == "zh":
        name = f"{last_name(language=language)}{first_name(language=language)}"
        return name
    else:
        raise ValueError(f"{language} language is not supported")


def get_email(name: str = "") -> str:
    """
    return a random email address
    """
    name = username(name)
    email_domains = [
        "126.com",
        "163.com",
        "qq.com",
        "sina.com",
        "sohu.com",
        "yahoo.com",
        "hotmail.com",
        "outlook.com",
        "gmail.com",
        "msn.com",
        "mail.com",
    ]
    return f"{name.lower()}@{random.choice(email_domains)}"


def get_md5(val: str = "") -> str:
    """Return an md5 hash of val, if no val then return a random md5 hash

    :param val: string, the value you want to md5 hash
    :returns: string, the md5 hash as a 32 char hex string
    """
    if not val:
        val = get_uuid()

    if getattr(val, "encode", None):
        ret = hashlib.md5(val.encode("utf-8")).hexdigest()
    else:
        ret = hashlib.md5(val).hexdigest()

    return ret


def get_uuid() -> str:
    """
    Generate a random UUID
    """
    return str(uuid.uuid4())


def get_int(min_size: int = 1, max_size=sys.maxsize) -> int:
    """
    return integer style data
    :param min_size:
    :param max_size:
    """
    return random.randint(min_size, max_size)


def get_int32(min_size: int = 1) -> int:
    """
    returns a 32-bit positive integer
    """
    return random.randint(min_size, 2**31-1)


def get_int64(min_size=1):
    """returns up to a 64-bit positive integer"""
    return random.randint(min_size, 2**63-1)


def get_float(min_size: float = None, max_size: float = None) -> float:
    """
    return a random float
    sames as the random method but automatically sets min and max

    :param min_size: float, the minimum float size you want
    :param max_size: float, the maximum float size you want
    :returns: float, a random value between min_size and max_size
    """
    float_info = sys.float_info
    if min_size is None:
        min_size = float_info.min
    if max_size is None:
        max_size = float_info.max
    return random.uniform(min_size, max_size)


def get_digits(count: int) -> str:
    """
    return a string value that contains count digits
    :param count: int, how many digits you want, so if you pass in 4, you would get
        4 digits
    :returns: string, this returns a string because the digits might start with
        zero
    """
    max_size = int("9" * count)
    ret = "{{:0>{}}}".format(count).format(get_int(0, max_size))
    return ret


def yes(specifier=0) -> int:
    """
    Decide if we should perform this action, this is just a simple way to do something
    I do in tests every now and again

    :Example:
        # EXAMPLE -- simple yes or no question
        if testdata.yes():
            # do this
        else:
            # don't do it

        # EXAMPLE -- multiple choice
        choice = testdata.yes(3)
        if choice == 1:
            # do the first thing
        elif choice == 2:
            # do the second thing
        else:
            # do the third thing

        # EXAMPLE -- do something 75% of the time
        if testdata.yes(0.75):
            # do it the majority of the time
        else:
            # but every once in a while don't do it

    :param specifier: int|float, if int, return a value between 1 and specifier.
        if float, return 1 approximately specifier percent of the time, return 0
        100% - specifier percent of the time
    :returns: integer, usually 1 (True) or 0 (False)
    """
    if specifier:
        if isinstance(specifier, int):
            choice = random.randint(1, specifier)

        else:
            if specifier < 1.0:
                specifier *= 100.0

            specifier = int(specifier)
            num = random.randint(0, 100)
            choice = 1 if num <= specifier else 0

    else:
        choice = random.choice([0, 1])

    return choice


def get_words(count: int = 0, as_str: bool = True, words=None) -> str:
    """get some amount of random words
    :param count: integer, how many words you want, 0 means a random amount (at most 20)
    :param as_str: boolean, True to return as string, false to return as list of words
    :param words: list, a list of words to choose from, defaults to unicode + ascii words
    :returns: unicode|list, your requested words
    """
    # since we specified we didn't care, randomly choose how many words there should be
    if count == 0:
        count = random.randint(1, 20)

    if not words:
        words = words_str

    ret_words = random.sample(words, count)
    return ret_words if not as_str else ' '.join(ret_words)


def get_word(words=None) -> str:
    return get_words(1, as_str=True, words=words)


def get_birthday(as_str: bool = False, start_age: int = 18, stop_age: int = 100) -> [str, datetime]:
    """
    return a random YYYY-MM-DD
    :param as_str: boolean, true to return the bday as a YYYY-MM-DD string
    :param start_age: int, minimum age of the birthday date
    :param stop_age: int, maximum age of the birthday date
    :returns: datetime.date|string
    """
    age = random.randint(start_age, stop_age)
    year = (datetime.datetime.utcnow() - datetime.timedelta(weeks=(age * 52))).year
    month = random.randint(1, 12)
    if month == 2:
        day = random.randint(1, 28)
    elif month in [9, 4, 6, 11]:
        day = random.randint(1, 30)
    else:
        day = random.randint(1, 31)

    birthday = datetime.date(year, month, day)
    if as_str:
        birthday = "{:%Y-%m-%d}".format(birthday)

    return birthday


def get_past_datetime(now=None, strftime=False) -> datetime:
    """
    a datetime guaranteed to be in the past from now.
    return: 2001-06-13 00:11:33.168502
    """
    if not now:
        now = datetime.datetime.now()
    if isinstance(now, datetime.timedelta):
        now = datetime.datetime.now() - now

    td = now - datetime.datetime(year=2000, month=1, day=1)
    data_time = now - datetime.timedelta(
        days=random.randint(1, max(td.days, 1)),
        seconds=random.randint(1, max(td.seconds, 1))
    )
    if strftime is True:
        return data_time.strftime("%Y-%m-%d %H:%M:%S")
    return data_time


def get_future_datetime(now=None, strftime=False) -> datetime:
    """
    a datetime guaranteed to be in the future from now
    return: 2034-02-23 04:59:41.168502
    """
    if not now:
        now = datetime.datetime.now()
    if isinstance(now, datetime.timedelta):
        now = datetime.datetime.now() + now
    data_time = now + datetime.timedelta(
        weeks=random.randint(1, 52 * 50),
        hours=random.randint(0, 24),
        days=random.randint(0, 365),
        seconds=random.randint(0, 86400)
    )
    if strftime is True:
        return data_time.strftime("%Y-%m-%d %H:%M:%S")
    return data_time


def get_now_datetime(strftime=False) -> [str, datetime]:
    """
    Get date time, default to current day。
    :return:
    """
    date_time = datetime.datetime.now()
    if strftime is True:
        return date_time.strftime("%Y-%m-%d %H:%M:%S")
    return date_time


def get_past_time() -> str:
    """
    Gets the past date time
    :return: 2019-05-16 00:34:30
    """
    number = random.randint(100000, 99999999)
    date_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time() - number))
    return date_time


def get_future_time() -> datetime:
    """
    Gets the future date time.
    :return: 2022-10-24 19:52:21
    """
    number = random.randint(100000, 99999999)
    date_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time() + number))
    return date_time


def get_date(day=None) -> str:
    """
    Get date, default to current day。
    :return:
    """
    if day is None:
        date = datetime.datetime.now().strftime("%Y-%m-%d")
    else:
        date = (datetime.datetime.now() + datetime.timedelta(days=day)).strftime("%Y-%m-%d")

    return date


def get_timestamp() -> str:
    """
    Get now timestamp
    @return: x ms
    """
    return str(int(time.time() * 1000))


def get_phone(operator: str = None) -> str:
    """
    get phone number
    :return:
    """
    if operator is None:
        all_operator = mobile + unicom + telecom
        top_third = random.choice(all_operator)
    elif operator == "mobile":
        top_third = random.choice(mobile)
    elif operator == "unicom":
        top_third = random.choice(unicom)
    elif operator == "telecom":
        top_third = random.choice(telecom)
    else:
        raise TypeError("Please select the right operator：'mobile'，'unicom'，'telecom' ")

    suffix = random.randint(9999999, 100000000)

    return "{}{}".format(top_third, suffix)



import inspect
import time

from kuto.utils.exceptions import ElementNameEmptyException, NoSuchElementException, DriverNotFound
from kuto.utils.log import logger
from kuto.utils.config import config
from kuto.core.ios.driver import IosDriver


class IosElem(object):
    """
    IOS原生元素定义
    """

    def __init__(self,
                 driver: IosDriver = None,
                 name: str = None,
                 label: str = None,
                 value: str = None,
                 text: str = None,
                 className: str = None,
                 xpath: str = None,
                 index: int = 0):
        """
        @param driver,
        @param name,
        param label,
        param value,
        param text,
        param className,
        param xpath,
        param index: 索引
        """
        # if driver is None:
        #     raise DriverNotFound('该控件未传入IOS driver参数')
        # else:
        #     self._driver = driver
        self._driver = driver

        self._kwargs = {}
        if name is not None:
            self._kwargs["name"] = name
        if label is not None:
            self._kwargs["label"] = label
        if value is not None:
            self._kwargs["value"] = value
        if text is not None:
            self._kwargs["text"] = text
        if className is not None:
            self._kwargs["className"] = className

        self._xpath = xpath
        self._index = index

    def __get__(self, instance, owner):
        if instance is None:
            return None

        self._driver = instance.driver
        return self

    # def error_handler(self):
    #     """异常处理，暂时只支持text"""
    #     errors = config.get_app('errors')
    #     if errors:
    #         logger.info(f'deal errors: {errors}')
    #         for error in errors:
    #             self._driver.d(**error).click_exists()

    # def _find_element(self, retry=3, timeout=3):
    #     """
    #     循环查找元素，查找失败先处理弹窗后重试
    #     @param retry: 重试次数
    #     @param timeout: 单次查找超时时间
    #     @return:
    #     """
    #     if self._xpath is not None:
    #         logger.info(f'find: xpath={self._xpath}')
    #     else:
    #         logger.info(f'find: {self._kwargs}[{self._index}]')
    #
    #     self._element = self._driver.d.xpath(self._xpath) if \
    #         self._xpath else self._driver.d(**self._kwargs)[self._index]
    #
    #     # 第一次查找
    #     try:
    #         if self._element.wait(timeout=timeout):
    #             return self._element
    #     except ConnectionError:
    #         logger.info('connect error, reconnecting.')
    #         # 由于WDA会意外链接错误
    #         self.driver = IosDriver(self.driver.device_id)
    #         time.sleep(5)
    #
    #     # 失败重试
    #     if retry > 0:
    #         for i in range(retry):
    #             self.error_handler()
    #             logger.info(f'try: {i + 1} times')
    #             try:
    #                 if self._element.wait(timeout=timeout):
    #                     return self._element
    #             except ConnectionError:
    #                 logger.info('connect error, reconnecting.')
    #                 # 由于WDA会意外链接错误
    #                 self.driver = IosDriver(self.driver.device_id)
    #                 time.sleep(5)
    #
    #     # 查找失败提示
    #     frame = inspect.currentframe().f_back
    #     caller = inspect.getframeinfo(frame)
    #     logger.warning(f'【{caller.function}:{caller.lineno}】find not {self._kwargs}')
    #     return None

    def get_element(self, timeout=5):
        """
        针对元素定位失败的情况，抛出NoSuchElementException异常
        @param timeout:
        @return:
        """
        if self._xpath is not None:
            logger.info(f'find: xpath={self._xpath}')
        else:
            logger.info(f'find: {self._kwargs}[{self._index}]')

        _element = self._driver.d.xpath(self._xpath) if \
            self._xpath else self._driver.d(**self._kwargs)[self._index]

        try:
            if _element.wait(timeout=timeout):
                return _element
            else:
                logger.info("loc fail")
                self._driver.screenshot("loc_fail")
                raise NoSuchElementException(f"[elem {self._kwargs} loc fail]")
        except ConnectionError:
            logger.info('connect error, reconnecting.')
            # 由于WDA会意外链接错误
            self._driver = IosDriver(self._driver.device_id, self._driver.pkg_name)
            time.sleep(5)

            logger.info('reconnected, retry find !!!')
            if _element.wait(timeout=timeout):
                return _element
            else:
                logger.info("loc fail")
                self._driver.screenshot("loc_fail")
                raise NoSuchElementException(f"[elem {self._kwargs} loc fail]")

    @property
    def info(self):
        """获取元素信息"""
        logger.info(f"get info")
        return self.get_element(timeout=1).info

    @property
    def text(self):
        """获取元素文本"""
        logger.info(f"get text")
        return self.get_element().text

    @property
    def bounds(self):
        """获取元素bounds属性"""
        logger.info(f"get bounds")
        return self.get_element().bounds

    @property
    def rect(self):
        """获取元素左上角坐标和宽高"""
        logger.info(f"get rect")
        return [item * self._driver.d.scale for item in list(self.get_element().bounds)]

    def exists(self, timeout=3):
        """
        判断元素是否存在当前页面
        @param timeout:
        @return:
        """
        logger.info(f"if exists")
        try:
            self.get_element(timeout=timeout)
        except:
            return False
        else:
            return True

    def click(self,):
        """
        单击
        @param: retry，重试次数
        @param: timeout，每次重试超时时间
        """
        logger.info('click')
        self.get_element().click()
        # if config.get_app('double_check'):
        #     logger.info('点击结果检查.')
        #     info_before = element.info
        #     element.click()
        #     time.sleep(1)
        #     # # 判断点击前后元素信息是否相同，如果相同就再点击一次
        #     if element.exists:
        #         if element.info == info_before:
        #             logger.debug('点击失败，再点一次')
        #             self.error_handler()
        #             element.click()
        # else:
        #     element.click()

    def click_exists(self, timeout=3):
        """元素存在时点击"""
        logger.info(f"click if exists")
        if self.exists(timeout=timeout):
            self.click()

    def clear(self):
        """清除文本"""
        logger.info("clear text")
        self.get_element().clear_text()

    def input(self, text):
        """输入内容"""
        logger.info(f"input {text}")
        self.get_element().set_text(text)

    # def scroll(self, direction=None):
    #     """
    #     滑动到元素可见的位置
    #     @param: direction，方向，"up", "down", "left", "right"
    #     @return:
    #     """
    #     logger.info(f"scroll {direction}")
    #     if direction is not None:
    #         self.get_element().scroll(direction=direction)
    #     else:
    #         self.get_element().scroll()
    #
    # def swipe_left(self):
    #     """往左滑动"""
    #     logger.info("swipe left")
    #     self.get_element().swipe("left")
    #
    # def swipe_right(self):
    #     """往右滑动"""
    #     logger.info("swipe right")
    #     self.get_element().swipe("right")
    #
    # def swipe_up(self):
    #     """往上滑动"""
    #     logger.info("swipe up")
    #     self.get_element().swipe("up")
    #
    # def swipe_down(self):
    #     """往下滑动"""
    #     logger.info("swipe down")
    #     self.get_element().swipe("down")


if __name__ == '__main__':
    pass







import inspect
import json
import os

import pytest

from kuto.utils.config import config
from kuto.utils.log import logger


class TestMain(object):
    """
    Support for app、web、http
    """
    def __init__(self,
                 did: str = None,
                 pkg: str = None,
                 browser: str = None,
                 headless: bool = False,
                 path: str = None,
                 rerun: int = 0,
                 xdist: bool = False,
                 host: str = None,
                 headers: dict = None,
                 state: dict = None
                 ):
        """
        @type state: 登录cookie，用于web
        @param did: 设备id，针对安卓和ios
        @param pkg: 应用包名，针对安卓和ios
        @param browser: 浏览器类型，chrome、firefox、webkit
        @param path: 用例目录，None默认代表当前文件
        @param rerun: 失败重试次数
        @param xdist: 并发支持
        @param host: 域名，针对接口和web
        @param headers: {
            "login": {},
            "visit": {}
        }
        """

        # 公共参数保存
        if browser:
            platform = "web"
        elif did:
            if "-" in did:
                platform = "ios"
            else:
                platform = "android"
        else:
            platform = "api"
        config.set_common("platform", platform)
        # api参数保存
        config.set_api("base_url", host)
        if headers:
            if 'login' not in headers.keys():
                raise KeyError("without login key!!!")
            login_ = headers.pop('login', {})
            config.set_api('login', login_)
            visit_ = headers.pop('visit', {})
            config.set_api('visit', visit_)
        # app参数保存
        config.set_app("device_id", did)
        config.set_app("pkg_name", pkg)
        # config.set_app("errors", errors)
        # web参数保存
        config.set_web("base_url", host)
        config.set_web("browser_name", browser)
        config.set_web("headless", headless)
        if state:
            config.set_web("state", json.dumps(state))

        # 执行用例
        logger.info('执行用例')
        if path is None:
            stack_t = inspect.stack()
            ins = inspect.getframeinfo(stack_t[1][0])
            file_dir = os.path.dirname(os.path.abspath(ins.filename))
            file_path = ins.filename
            if "\\" in file_path:
                this_file = file_path.split("\\")[-1]
            elif "/" in file_path:
                this_file = file_path.split("/")[-1]
            else:
                this_file = file_path
            path = os.path.join(file_dir, this_file)
        cmd_list = [
            '-sv',
            '--reruns', str(rerun),
            '--alluredir', 'reports', '--clean-alluredir'
        ]
        if path:
            cmd_list.insert(0, path)
        if xdist:
            """仅支持http接口测试和web测试，并发基于每个测试类，测试类内部还是串行执行"""
            cmd_list.insert(1, '-n')
            cmd_list.insert(2, 'auto')
        logger.info(cmd_list)
        pytest.main(cmd_list)

        # 公共参数保存
        config.set_common("platform", "api")
        # api参数保存
        config.set_api("base_url", None)
        config.set_api('login', {})
        config.set_api('visit', {})
        # app参数保存
        config.set_app("device_id", None)
        config.set_app("pkg_name", None)
        # config.set_app("errors", [])
        # web参数保存
        config.set_web("base_url", None)
        config.set_web("browser_name", "chrome")
        config.set_web("headless", False)
        config.set_web("state", None)


main = TestMain


if __name__ == '__main__':
    main()




import kuto
from kuto import AdrElem


class HomePage(kuto.Page):
    """首页"""
    ad = AdrElem(resourceId='com.qizhidao.clientapp:id/bottom_btn')
    my = AdrElem(text='我的')


class MyPage(kuto.Page):
    """我的页"""
    setting = AdrElem(resourceId='com.qizhidao.clientapp:id/me_top_bar_setting_iv')


class TestSetting(kuto.TestCase):
    """设置页测试"""

    def start(self):
        self.home_page = HomePage(self.d)
        self.my_page = MyPage(self.d)

    def test_1(self):
        """进入设置页"""
        self.home_page.ad.click_exists(timeout=5)
        self.home_page.my.click()
        self.my_page.setting.click()


if __name__ == '__main__':
    kuto.main(did='UJK0220521066836', pkg='com.qizhidao.clientapp')




import json
import time
from typing import Union
from urllib import parse

from playwright.async_api import Locator

from kuto.core.android.driver import AndroidDriver
from kuto.core.api.request import HttpReq
from kuto.core.ios.driver import IosDriver
from kuto.utils.config import config
from kuto.utils.log import logger
from kuto.utils.exceptions import (
    HostIsNull
)
from kuto.core.web.driver import PlayWrightDriver


class Page(object):
    """页面基类，用于pom模式封装"""

    def __init__(self, driver: Union[AndroidDriver, IosDriver]):
        self.driver = driver

    @staticmethod
    def sleep(n):
        """休眠"""
        logger.info(f"休眠 {n} 秒")
        time.sleep(n)


class TestCase(HttpReq):
    """
    测试用例基类，所有测试用例需要继承该类
    """

    d: Union[AndroidDriver, IosDriver, PlayWrightDriver] = None

    # ---------------------初始化-------------------------------
    def start_class(self):
        """
        Hook method for setup_class fixture
        :return:
        """
        pass

    def end_class(self):
        """
        Hook method for teardown_class fixture
        :return:
        """
        pass

    @classmethod
    def setup_class(cls):
        cls().start_class()

    @classmethod
    def teardown_class(cls):
        cls().end_class()

    def start(self):
        """
        Hook method for setup_method fixture
        :return:
        """
        pass

    def end(self):
        """
        Hook method for teardown_method fixture
        :return:
        """
        pass

    def setup_method(self):
        self.start_time = time.time()
        platform = config.get_common("platform")
        if platform == "android":
            device_id = config.get_app("device_id")
            pkg_name = config.get_app("pkg_name")
            self.d = AndroidDriver(device_id, pkg_name)
        elif platform == "ios":
            device_id = config.get_app("device_id")
            pkg_name = config.get_app("pkg_name")
            self.d = IosDriver(device_id, pkg_name)
        elif platform == "web":
            browserName = config.get_web("browser_name")
            headless = config.get_web("headless")
            state = config.get_web("state")
            if state:
                state_json = json.loads(state)
                self.d = PlayWrightDriver(browserName=browserName, headless=headless, state=state_json)
            else:
                self.d = PlayWrightDriver(browserName=browserName, headless=headless)
        if isinstance(self.d, (AndroidDriver, IosDriver)):
            self.d.start_app()
        self.start()

    def teardown_method(self):
        self.end()
        if isinstance(self.d, PlayWrightDriver) == "web":
            self.d.close()
        if isinstance(self.d, (AndroidDriver, IosDriver)):
            self.d.stop_app()
        take_time = time.time() - self.start_time
        logger.debug("[run_time]: {:.2f} s".format(take_time))

    @staticmethod
    def sleep(n: float):
        """休眠"""
        logger.debug(f"sleep: {n}s")
        time.sleep(n)

    def open(self, url=None):
        """浏览器打开页面"""
        # 拼接域名
        if url is None:
            base_url = config.get_web("base_url")
            if not base_url:
                raise HostIsNull('base_url is null')
            url = base_url
        else:
            if "http" not in url:
                base_url = config.get_web("base_url")
                if not base_url:
                    raise HostIsNull('base_url is null')
                url = parse.urljoin(base_url, url)

        # 访问页面
        self.d.visit(url)

    def newPage(self, locator: Locator):
        """切换到新页签"""
        self.d.switch_tab(locator)



import time

from kuto.utils.log import logger
from kuto.core.ocr.ocr_discern import OCRDiscern
from kuto.utils.exceptions import ElementNameEmptyException


class OCRElem(object):
    """ocr识别定位"""

    def __init__(self, driver=None, text: str = None, desc: str = None):
        self.driver = driver
        self.text = text
        if desc is None:
            raise ElementNameEmptyException("请设置控件名称")
        else:
            self._desc = desc

    def __get__(self, instance, owner):
        if instance is None:
            return None

        self.driver = instance.driver
        return self

    def exists(self, retry=3, timeout=1):
        logger.info(f'ocr识别文本: {self.text} 是否存在')
        time.sleep(3)
        for i in range(retry):
            logger.info(f'第{i+1}次查找:')
            self.driver.screenshot('SourceImage.png', with_time=False)
            res = OCRDiscern().get_coordinate(self.text)
            if isinstance(res, tuple):
                self.driver.screenshot(f'ocr识别定位-{self.text}')
                return True
            time.sleep(timeout)
        else:
            self.driver.screenshot(f'ocr识别定位失败-{self.text}')
            return False

    def click(self, retry=3, timeout=1):
        logger.info(f'ocr点击文本: {self.text}')
        time.sleep(3)
        for i in range(retry):
            logger.info(f'第{i+1}次查找:')
            self.driver.screenshot('SourceImage.png', with_time=False)
            res = OCRDiscern().get_coordinate(self.text)
            if isinstance(res, tuple):
                logger.info(f'识别坐标为: {res}')
                self.driver.screenshot(f'ocr识别定位-{self.text}')
                self.driver.click(res[0], res[1])
                return
            time.sleep(timeout)
        else:
            self.driver.screenshot(f'ocr识别定位失败-{self.text}')
            raise Exception('通过OCR未识别指定文字或置信度过低，无法进行点击操作！')


if __name__ == '__main__':
    from kuto.core.android.driver import AndroidDriver

    driver = AndroidDriver()
    driver.pkg_name = 'com.qizhidao.clientapp'
    driver.start_app()
    elem = OCRElem(driver, '查企业', '查企业入口')
    elem.click()




import click
import subprocess
# import argparse

# from webdriver_manager.firefox import GeckoDriverManager
# from webdriver_manager.microsoft import EdgeChromiumDriverManager, IEDriverManager
#
# from kuto.utils.log import logger
# # from kuto.scaffold import create_scaffold
from kuto import __version__, __description__
# from kuto.utils.webdriver_manager_extend import ChromeDriverManager
from kuto.scaffold import create_scaffold


@click.command()
@click.version_option(version=__version__, help="Show version.")
@click.option("-i", "--install", is_flag=True, help="Install the browser driver.")
@click.option("-p", "--platform", type=str, default=None, help="Create demo by platform, api、ios、android、web")
def main(install, platform):
    """API test: parse command line options and run commands."""
    if install:
        subprocess.run(["playwright", "install"])

    if platform:
        create_scaffold(platform)

    # parser = argparse.ArgumentParser(description=__description__)
    # parser.add_argument(
    #     "-v", "--version", dest="version", action="store_true", help="版本号"
    # )
    # parser.add_argument(
    #     "-p", "--platform", dest="platform", help="所属平台，android、ios、web、api，用于创建demo"
    # )
    # parser.add_argument(
    #     "-i", "--install", dest="install", help="浏览器驱动名称，chrome、firefox、edge，用于安装浏览器驱动"
    # )

    # args = parser.parse_args()
    # version = args.version
    # platform = args.platform
    # install = args.install

    # if version:
    #     print(__version__)
    #     return 0
    # if platform:
    #     create_scaffold(platform)
    #     return 0
    # if install:
    #     install_driver(install)
    #     return 0


# def install_driver(browser: str) -> None:
#     """
#     Download and install the browser driver
#     :param browser: The Driver to download. Pass as `chrome/firefox/ie/edge`. Default Chrome.
#     :return:
#     """
#
#     if browser == "chrome":
#         driver_path = ChromeDriverManager().install()
#         logger.info(f"Chrome Driver[==>] {driver_path}")
#     elif browser == "firefox":
#         driver_path = GeckoDriverManager().install()
#         logger.info(f"Firefox Driver[==>] {driver_path}")
#     elif browser == "ie":
#         driver_path = IEDriverManager().install()
#         logger.info(f"IE Driver[==>] {driver_path}")
#     elif browser == "edge":
#         driver_path = EdgeChromiumDriverManager().install()
#         logger.info(f"Edge Driver[==>] {driver_path}")
#     else:
#         raise NameError(f"Not found '{browser}' browser driver.")



"""
@Author: kang.yang
@Date: 2023/5/13 10:16
"""
import time

from kuto.utils.exceptions import NoSuchElementException
from kuto.utils.log import logger
from kuto.core.web.driver import PlayWrightDriver
from playwright.sync_api import expect


class WebElem:
    """
    通过selenium定位的web元素
    """

    def __init__(self,
                 driver: PlayWrightDriver = None,
                 xpath: str = None,
                 css: str = None,
                 text: str = None,
                 placeholder: str = None,
                 index: int = None
                 ):
        """

        @param driver: 浏览器驱动
        @param xpath: 根据xpath进行定位
        @param css: 根据css selector进行定位
        @param text:
        @param placeholder:
        """
        self._driver = driver
        self._xpath = xpath
        self._css = css
        self._text = text
        self._placeholder = placeholder
        self._index = index

    def __get__(self, instance, owner):
        if instance is None:
            return None

        self._driver = instance.driver
        return self

    def find_element(self):
        element = None
        if self._text:
            logger.info(f"try find text: {self._text}")
            element = self._driver.page.get_by_text(self._text)
        if self._placeholder:
            logger.info(f"try find placeholder: {self._placeholder}")
            element = self._driver.page.get_by_placeholder(self._placeholder)
        if self._css:
            logger.info(f"try find css: {self._css}")
            element = self._driver.page.locator(self._css)
        if self._xpath:
            logger.info(f"try find xpath: {self._xpath}")
            element = self._driver.page.locator(self._xpath)
        if self._index:
            element = element.nth(self._index)
        return element

    def fail_handler(self):
        logger.info("loc fail")
        self._driver.screenshot("loc_fail")
        raise NoSuchElementException(f"[elem loc fail]")

    def click(self, timeout=10):
        logger.info("click")
        try:
            self.find_element().click(timeout=timeout*1000)
        except:
            self.fail_handler()

    def input(self, text, timeout=10):
        logger.info("input")
        try:
            self.find_element().fill(text, timeout=timeout*1000)
        except:
            self.fail_handler()

    def check(self, timeout=10):
        logger.info("check")
        try:
            self.find_element().check(timeout=timeout*1000)
        except:
            self.fail_handler()

    def select(self, value: str, timeout=10):
        logger.info("select")
        try:
            self.find_element().select_option(value, timeout=timeout*1000)
        except:
            self.fail_handler()

    def text(self):
        logger.info("get text")
        return self.find_element().text_content()

    def assert_visible(self):
        logger.info("assert visible")
        expect(self.find_element()).to_be_visible()

    def assert_hidden(self):
        logger.info("assert hidden")
        expect(self.find_element()).to_be_hidden()

    def assert_contain(self, text: str):
        logger.info("assert contain text")
        expect(self.find_element()).to_contain_text(text)

    def assert_have_text(self, text: str):
        logger.info("assert have text")
        expect(self.find_element()).to_have_text(text)


class FraElem(WebElem):

    def __init__(self, driver: PlayWrightDriver = None, frame: str = None, xpath: str = None, css: str = None,
                 text: str = None, placeholder: str = None, index: int = None):
        """

        @param driver: 浏览器驱动
        @type frame: frame定位方式，使用正常的css定位方式即可
        @param xpath: 根据xpath进行定位
        @param css: 根据css selector进行定位
        @param text: 根据标签的文本定位
        @param placeholder: 根据输入框的placeholder定位
        """
        super().__init__(driver, xpath, css, text, placeholder, index)
        self._driver = driver
        self._frame_loc = frame
        self._xpath = xpath
        self._css = css
        self._text = text
        self._placeholder = placeholder
        self._index = index

    def find_element(self):

        element = None
        if self._text:
            logger.info(f"try find text: {self._text} in iframe {self._frame_loc}")
            element = self._driver.page.frame_locator(self._frame_loc).get_by_text(self._text)
        if self._placeholder:
            logger.info(f"try find placeholder: {self._placeholder} in iframe {self._frame_loc}")
            element = self._driver.page.frame_locator(self._frame_loc).get_by_placeholder(self._placeholder)
        if self._css:
            logger.info(f"try find css: {self._css} in iframe {self._frame_loc}")
            element = self._driver.page.frame_locator(self._frame_loc).locator(self._css)
        if self._xpath:
            logger.info(f"try find xpath: {self._xpath} in iframe {self._frame_loc}")
            element = self._driver.page.frame_locator(self._frame_loc).locator(self._xpath)
        if self._index:
            element = element.nth(self._index)
        return element



import json
import yaml

from kuto.utils.excel import Excel, CSV


def read_json(file_path, key=None):
    """
    读取json文件中的指定key
    @return
    """
    with open(file_path, 'r', encoding='utf-8') as f:
        json_data = json.load(f)
    if isinstance(json_data, list):
        return json_data
    else:
        if key:
            return json_data[key]
        raise ValueError('key不能为空')


def read_yaml(file_path, key=None):
    """
    读取yaml文件中的指定key
    @param file_path:
    @param key:
    """
    with open(file_path, 'r', encoding='utf-8') as f:
        yaml_data = yaml.load(f, Loader=yaml.FullLoader)
    if isinstance(yaml_data, list):
        return yaml_data
    else:
        if key:
            return yaml_data[key]
        raise ValueError('key不能为空')


def read_csv(file_path, row=None):
    """
    读取csv文件中的指定行
    @param file_path: 文件名
    @param row: 行数，从1开始
    """
    csv_file = CSV(file_path)
    if row:
        csv_data = [csv_file.read_row_index(row)]
    else:
        csv_data = csv_file.read_all()
    return csv_data


def read_excel(file_path, row=None):
    """
    读取excel文件中的指定行，暂时只支持读取第一个sheet
    @param file_path: 文件名
    @param row: 行数，从1开始
    """
    excel_file = Excel(file_path)
    if row:
        excel_data = [excel_file.read_row_index(row)]
    else:
        excel_data = excel_file.read_all()
    return excel_data


if __name__ == '__main__':
    print(read_json('data.json', key='names'))
    print(read_yaml('data.yml', key='names'))
    print(read_csv('data.csv', row=1))
    print(read_excel('data.xlsx', row=1))






import kuto


class TestHome(kuto.TestCase):

    def test_1(self):
        """获取金刚位列表"""
        payload = {"type": 2}
        headers = {"user-agent-web": "X/b67aaff2200d4fc2a2e5a079abe78cc6"}
        self.post('/qzd-bff-app/qzd/v1/home/getToolCardListForPc', json=payload, headers=headers)
        self.assertEq('code', 0)


if __name__ == '__main__':
    kuto.main(host='https://app.qizhidao.com')



import re
import sys

"""
https://www.ssa.gov/oact/babynames/decades/names2010s.html
"""

en_first_names_male = list(set(re.split(r"\s+", """
Noah Liam Jacob William Mason Ethan Michael Alexander James Elijah Benjamin Daniel Aiden Logan Jayden 
Matthew Lucas David Jackson Joseph Anthony Samuel Joshua Gabriel Andrew John Christopher Oliver Dylan 
Carter Isaac Luke Henry Owen Ryan Nathan Wyatt Caleb Sebastian Jack Christian Jonathan Julian Landon 
Levi Isaiah Hunter Aaron Charles Thomas Eli Jaxon Connor Nicholas Jeremiah Grayson Cameron Brayden Adrian 
Evan Jordan Josiah Angel Robert Gavin Tyler Austin Colton Jose Dominic Brandon Ian Lincoln Hudson Kevin 
Zachary Adam Mateo Jason Chase Nolan Ayden Cooper Parker Xavier Asher Carson Jace Easton Justin Leo 
Bentley Jaxson Nathaniel Blake Elias Theodore Kayden Luis Tristan Bryson Ezra Juan Brody Vincent Micah 
Miles Santiago Cole Ryder Carlos Damian Leonardo Roman Max Sawyer Jesus Diego Greyson Alex Maxwell Axel 
Eric Wesley Declan Giovanni Ezekiel Braxton Ashton Ivan Hayden Camden Silas Bryce Weston Harrison Jameson 
George Antonio Timothy Kaiden Jonah Everett Miguel Steven Richard Emmett Victor Kaleb Kai Maverick Joel 
Bryan Maddox Kingston Aidan Patrick Edward Emmanuel Jude Alejandro Preston Luca Bennett Jesse Colin Jaden 
Malachi Kaden Jayce Alan Kyle Marcus Brian Ryker Grant Jeremy Abel Riley Calvin Brantley Caden Oscar Abraham 
Brady Sean Jake Tucker Nicolas Mark Amir Avery King Gael Kenneth Bradley Cayden Xander Graham Rowan 
""".strip())))

en_first_names_female = list(set(re.split(r"\s+", """
Emma Olivia Sophia Isabella Ava Mia Abigail Emily Charlotte Madison Elizabeth Amelia Evelyn Ella Chloe 
Harper Avery Sofia Grace Addison Victoria Lily Natalie Aubrey Lillian Zoey Hannah Layla Brooklyn Scarlett 
Zoe Camila Samantha Riley Leah Aria Savannah Audrey Anna Allison Gabriella Claire Hailey Penelope Aaliyah 
Sarah Nevaeh Kaylee Stella Mila Nora Ellie Bella Lucy Alexa Arianna Violet Ariana Genesis Alexis Eleanor 
Maya Caroline Peyton Skylar Madelyn Serenity Kennedy Taylor Alyssa Autumn Paisley Ashley Brianna Sadie 
Naomi Kylie Julia Sophie Mackenzie Eva Gianna Luna Katherine Hazel Khloe Ruby Melanie Piper Lydia Aubree 
Madeline Aurora Faith Alexandra Alice Kayla Jasmine Maria Annabelle Lauren Reagan Elena Rylee Isabelle 
Bailey Eliana Sydney Makayla Cora Morgan Natalia Kimberly Vivian Quinn Valentina Andrea Willow Clara London 
Jade Liliana Jocelyn Trinity Kinsley Brielle Mary Molly Hadley Delilah Emilia Josephine Brooke Ivy Lilly 
Adeline Payton Lyla Isla Jordyn Paige Isabel Mariah Mya Nicole Valeria Destiny Rachel Ximena Emery Everly 
Sara Angelina Adalynn Kendall Reese Aliyah Margaret Juliana Melody Amy Eden Mckenzie Laila Vanessa Ariel 
Gracie Valerie Adalyn Brooklynn Gabrielle Kaitlyn Athena Elise Jessica Adriana Leilani Ryleigh Daisy Nova 
Norah Eliza Rose Rebecca Michelle Alaina Catherine Londyn Summer Lila Jayla Katelyn Daniela Harmony Alana 
Amaya Emerson Julianna Cecilia Izabella""".strip())))

en_last_names = list(set(re.split(r"\s+", """
Smith Johnson Williams Jones Brown Davis Miller Wilson Moore Taylor Anderson Thomas Jackson White Harris 
Martin Thompson Garcia Martinez Robinson Clark Rodriguez Lewis Lee Walker Hall Allen Young Hernandez King 
Wright Lopez Hill Scott Green Adams Baker Gonzalez Nelson Carter Mitchell Perez Roberts Turner Phillips 
Campbell Parker Evans Edwards Collins Stewart Sanchez Morris Rogers Reed Cook Morgan Bell Murphy Bailey 
Rivera Cooper Richardson Cox Howard Ward Torres Peterson Gray Ramirez James Watson Brooks Kelly Sanders 
Price Bennett Wood Barnes Ross Henderson Coleman Jenkins Perry Powell Long Patterson Hughes Flores Washington 
Butler Simmons Foster Gonzales Bryant Alexander Russell Griffin Diaz Hayes Myers Ford Hamilton Graham Sullivan 
Wallace Woods Cole West Jordan Owens Reynolds Fisher Ellis Harrison Gibson Mcdonald Cruz Marshall Ortiz Gomez 
Murray Freeman Wells Webb Simpson Stevens Tucker Porter Hunter Hicks Crawford Henry Boyd Mason Morales Kennedy 
Warren Dixon Ramos Reyes Burns Gordon Shaw Holmes Rice Robertson Hunt Black Daniels Palmer Mills Nichols Grant 
Knight Ferguson Rose Stone Hawkins Dunn Perkins Hudson Spencer Gardner Stephens Payne Pierce Berry Matthews 
Arnold Wagner Willis Ray Watkins Olson Carroll Duncan Snyder Hart Cunningham Bradley Lane Andrews Ruiz Harper 
Fox Riley Armstrong Carpenter Weaver Greene Lawrence Elliott Chavez Sims Austin Peters Kelley Franklin 
""".strip())))

zh_names_male = list(set(re.split(r"\s+", """德义 苍 鹏云 炎 和志 新霁 澜 星泽 驰轩 楚 宏深 全 波涛 飞文 波 振国 凯 光启 经略 乐天 
志强 作人 英叡 英华 星阑 景龙 鹏鲸 采 浩然 举 芬 鸿才 卫 嘉纳 旭东 玉泽 祺瑞 荫 茂德 博 鸿羲 彦 涵衍 开诚 鸿远 凯歌 星华 玉宇 潍 德华 甲 
梓 正阳 文乐 高杰 骄 腾逸 鸿畅 修平 飞扬 宏爽 乐康 和风 洁 令羽 承载 礼 昊硕 天瑞 安宁 高义 兴朝 颖 浩波 洲 颉 良骏 颜 锋 承业 彭泽 骏年 
坚白 运 承悦 钧 涵蓄 修雅 濮 天翰 经纬 天工 国源 奇迈 海逸 郁 俊侠 烨伟 昆峰 高旻 高超 鹏运 安 浩初 英逸 英豪 元甲 弘雅 温瑜 高雅 德明 慈 
良畴 良骥 康德 皓轩 涵忍 振锐 嘉石 浦泽 飞飙 子 安澜 奇略 英 学海 悦 明喆 兴昌 凯安 乐生 仕 凯泽 和煦 鸿才 伟懋 华美 怀 景行 鸿远 宜人 顺 
彭魄 宇文 景中 浩壤 奕 乐水 俊捷 高阳 云天 豪 驰翰 鑫鹏 鸾 伟诚 彤 华皓 安翔 正德 兴运 鸿达 硕 光济 博明 彭勃 阳曦 熠彤 泽语 睿德 原 令璟 
璞玉 明珠 许 宸 景明 春 子明 和昶 远航 枫 宜民 修然 巍然 卿 睿思 兴思 朔 高芬 容 长 鸿信 晨濡 俊哲 和悦 俊发 文曜 伟兆 昊天 宾""".strip())))

zh_names_female = list(set(re.split(r"\s+", """海莹 曼珠 虹影 凝安 淳美 清润 旋 馨香 骊霞 水丹 长文 怀薇 平卉 向露 秀敏 青柏 尔阳 奥婷 
智美 雅可 骊燕 燕珺 白曼 春枫 谷之 暖姝 易绿 娅欣 欢 半梅 忆彤 宇 茗 芳洁 双文 艳芳 珍丽 杨 若星 松 葳 晓畅 菱华 新荣 觅露 冰夏 初柳 迎蕾 
海宁 香 妙颜 靖之 冰莹 天菱 诗丹 思思 玄素 安波 依秋 香巧 蕙 朝旭 怡 赞悦 梓彤 婉静 庄静 冬卉 冷雪 冰海 吟怀 月灵 优瑗 清嘉 悦爱 迎荷 旎旎 
秋柳 巧春 美偲 忆灵 谷兰 雅娴 靖易 曼冬 溶溶 冷之 幼 芳蔼 妃 惜蕊 曼彤 傲之 愫 雪儿 以轩 丹秋 格 若云 骏 雅洁 朝雨 合美 馥 绿柳 慕凝 静曼 
品韵 易容 娅芳 月怡 琲 如云 格格 溪 儿 璠瑜 丁辰 秀媚 岚岚 筱 听 元冬 白山 乃 茉 寄蕾 倚 傲儿 谷 淑君 帅 凡灵 语林 叶 幻珊 山雁 涵涵 灿 
绮梅 平宁 天真 迎丝 水蓉 灵韵 甜恬 盼盼 韵宁 渟 安荷 智 虹英 访烟 玲玲 正思 霏 寄灵 友琴 绿柏 觅儿 娅玟 书琴 天蓉 宝 巧 寒雁 云韶 青旋 凝安 
清懿 依云 以松 妙珍 灵阳 韶美 梓珊 未 初雪 姝艳 采梦 苒 若山 绚 彤彤 家 春娇 梦云 溪蓝 以松 半烟 孤云 玑 元绿 如曼 痴瑶 灵雨 梦寒 湘云 涵畅 
玲琳 素华 """.strip())))

zh_last_name = list(set(re.split(r"\s+", """赵 钱 孙 李 周 吴 郑 王 冯 陈 褚 卫 蒋 沈 韩 杨 朱 秦 尤 许 何 吕 施 张 孔 曹 严 华 金 
魏 陶 姜 戚 谢 邹 喻 柏 水 窦 章 云 苏 潘 葛 奚 范 彭 郎 鲁 韦 昌 马 苗 凤 花 方 俞 任 袁 柳 酆 鲍 史 唐 费 廉 岑 薛 雷 贺 倪 汤 
滕 殷 罗 毕 郝 邬 安 常 乐 于 时 傅 皮 卞 齐 康 伍 余 元 卜 顾 孟 平 黄 和 萧 尹 湛 汪 祁 毛 禹 狄 米 贝 成 戴 谈 宋 茅 庞 熊 纪 
舒 屈 项 祝 董 梁 杜 阮 蓝 闵 席 季 麻 强 贾 路 娄 危 江 童 颜 郭 梅 盛 林 刁 钟 徐 邱 骆 高 夏 蔡 田 樊 胡 凌 霍 虞 万 支 柯 昝 
管 卢 莫 白 房 裘 缪 干 解 应 宗 丁 宣 贲 邓 郁 单 杭 洪 包 诸 左 石 崔 吉 钮 龚 程 嵇 邢 滑 裴 陆 荣 翁 荀 羊 宇文 尉迟 延陵 羊舌 
羊角 乐正 诸葛 颛孙 仲孙 仲长 长孙 钟离 宗政 左丘 主父 宰父 子书 子车 子桑 百里 北堂 北野 哥舒 谷梁 闻人 王孙 王官 王叔 巫马 微生 淳于 
单于 成公 叱干 叱利 褚师 端木 东方 东郭 东宫 东野 东里 东门 第二 第五 公祖 公玉 公西 公孟 公伯 公仲 公孙 公广 公上 公冶 公羊 公良 公户 
公仪 公山 公门 公坚 公乘 欧阳 濮阳 青阳 漆雕 壤驷 上官 司徒 司马 司空 司寇 士孙 申屠 叔孙 叔仲 侍其 令狐 梁丘 闾丘 刘傅 慕容 万俟 谷利 
高堂 南宫 南门 南荣 南野 女娲 纳兰 澹台 拓跋 太史 太叔 太公 秃发 夏侯 西门 鲜于 轩辕 相里 皇甫 赫连 呼延 胡母 亓官 夹谷 即墨 独孤 段干 
达奚""".strip())))

# via: http://www.lipsum.com/feed/html
# russian is from: http://masterrussian.com/vocabulary/most_common_words.htm
# japanese (4bytes) are from: http://www.i18nguy.com/unicode/supplementary-test.html
ascii_paragraphs = '''
Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Phasellus
pharetra urna sit amet magna. Donec posuere porta velit. Vestibulum sed libero.
Ut vestibulum sodales arcu. Proin vulputate, mi quis luctus ornare, elit ligula fringilla nisi,
eu tempor purus felis a enim. Phasellus in justo et nisi rhoncus porttitor. Donec ligula felis,
sagittis at, vestibulum eu, vehicula sed, nisl. Aenean convallis pharetra nisl. Mauris imperdiet
libero eu urna ultrices vulputate. Donec semper nunc et nibh. In hac habitasse platea dictumst.
Fusce et ipsum semper velit tempor pharetra. Donec pretium sollicitudin purus. Cras mi velit,
egestas id, ultrices vitae, viverra sit amet, justo.

Quisque cursus tristique nunc. Fusce varius, orci et pellentesque aliquet,
nibh ipsum sodales lorem, iaculis tincidunt massa metus ut erat. Fusce dictum,
dolor ut laoreet aliquam, massa urna placerat nibh, vitae tristique nisl neque posuere mi.
Aliquam at orci. Nulla sem. Nullam risus. Nullam pharetra dapibus mauris. Mauris mollis pretium arcu.
Vestibulum sem massa, tempor a, dictum id, rutrum eu, ligula. Class aptent taciti sociosqu ad
litora torquent per conubia nostra, per inceptos himenaeos. Curabitur ultrices dignissim nibh.
Aenean nisl.

Integer bibendum pharetra orci. Suspendisse commodo, lorem elementum egestas hendrerit,
metus elit rutrum sapien, quis aliquam nibh nisi at ligula. Nam lobortis commodo mauris.
Vivamus semper, leo vel accumsan mattis, nulla elit vestibulum augue, vitae pharetra dolor nibh
sit amet odio. Pellentesque scelerisque ipsum id elit. Nulla aliquet semper dolor. Praesent ut lorem.
Curabitur dictum, magna eu porttitor rutrum, ipsum justo porttitor erat, sit amet tristique est ante
ut elit. Mauris vel est. In cursus, velit quis pharetra adipiscing, purus quam sagittis mi,
eget molestie leo lectus ac lacus. Curabitur ante massa, aliquam ut, scelerisque a, condimentum at,
eros. Nunc vitae neque. Nam sagittis scelerisque magna. Class aptent taciti sociosqu ad litora
torquent per conubia nostra, per inceptos himenaeos. Donec cursus pede. Quisque a mauris nec
turpis convallis scelerisque. Donec quam lorem, mollis vestibulum, euismod in, hendrerit et, sapien.
Curabitur felis.

Morbi pretium lorem imperdiet dui. Maecenas quis ligula. Morbi tempor velit sit amet felis.
Donec at dui. Donec neque. Quisque quis mauris a libero ultrices iaculis. Integer congue feugiat justo.
Quisque imperdiet lectus eu orci. Class aptent taciti sociosqu ad litora torquent per conubia nostra,
per inceptos himenaeos. Vivamus id lectus. Phasellus odio nisi, auctor eu, hendrerit quis,
iaculis sit amet, felis. Sed blandit mollis nunc. Sed velit magna, tristique tristique, porttitor ut,
dictum a, arcu. In hac habitasse platea dictumst. Cras semper bibendum tortor. Cum sociis natoque
penatibus et magnis dis parturient montes, nascetur ridiculus mus. Suspendisse potenti.
In hac habitasse platea dictumst. Fusce mi sem, varius vitae, molestie ut, gravida venenatis, nibh.
Nam risus lectus, interdum at, condimentum eu, aliquet et, ipsum.

Mauris mi tortor, elementum ut, mattis eget, aliquam a, tellus.
Suspendisse porttitor orci. Donec rutrum diam non est. Duis ac nunc. Cras sollicitudin aliquet mi.
Cras in pede. Vestibulum ante ipsum primis in faucibus orci luctus et ultrices posuere cubilia Curae;
Nam vehicula est at metus. Suspendisse sapien. Nunc lobortis tortor sed purus hendrerit pellentesque.
Nunc laoreet. Morbi pharetra. Integer cursus molestie turpis. Nam cursus sodales sem.
Maecenas non lacus. Pellentesque habitant morbi tristique senectus et netus et malesuada fames
ac turpis egestas. Nam vel nibh eu nulla blandit facilisis. Sed varius turpis ac neque.
Curabitur vel erat. Morbi sed purus id erat tincidunt ullamcorper.
'''

unicode_paragraphs = '''
\u0437\u043d\u0430\u0442\u044c \u043c\u043e\u0439 \u0434\u043e \u0438\u043b\u0438 \u0435\u0441\u043b\u0438
\u0432\u0440\u0435\u043c\u044f \u0440\u0443\u043a\u0430 \u043d\u0435\u0442 \u0441\u0430\u043c\u044b\u0439
\u043d\u0438 \u0441\u0442\u0430\u0442\u044c \u0431\u043e\u043b\u044c\u0448\u043e\u0439 \u0434\u0430\u0436\u0435
\u0434\u0440\u0443\u0433\u043e\u0439 \u043d\u0430\u0448 \u0441\u0432\u043e\u0439 \u043d\u0443 \u043f\u043e\u0434
\u0433\u0434\u0435 \u0434\u0435\u043b\u043e \u0435\u0441\u0442\u044c \u0441\u0430\u043c \u0440\u0430\u0437
\u0447\u0442\u043e\u0431\u044b \u0434\u0432\u0430 \u0442\u0430\u043c \u0447\u0435\u043c \u0433\u043b\u0430\u0437
\u0436\u0438\u0437\u043d\u044c \u043f\u0435\u0440\u0432\u044b\u0439 \u0434\u0435\u043d\u044c \u0442\u0443\u0442
\u0432\u043e \u043d\u0438\u0447\u0442\u043e \u043f\u043e\u0442\u043e\u043c \u043e\u0447\u0435\u043d\u044c
\u0441\u043e \u0445\u043e\u0442\u0435\u0442\u044c \u043b\u0438 \u043f\u0440\u0438 \u0433\u043e\u043b\u043e\u0432\u0430
\u043d\u0430\u0434\u043e \u0431\u0435\u0437 \u0432\u0438\u0434\u0435\u0442\u044c \u0438\u0434\u0442\u0438
\u0442\u0435\u043f\u0435\u0440\u044c \u0442\u043e\u0436\u0435 \u0441\u0442\u043e\u044f\u0442\u044c
\u0434\u0440\u0443\u0433 \u0434\u043e\u043c \u0441\u0435\u0439\u0447\u0430\u0441 \u043c\u043e\u0436\u043d\u043e
\u043f\u043e\u0441\u043b\u0435 \u0441\u043b\u043e\u0432\u043e \u0437\u0434\u0435\u0441\u044c
\u0434\u0443\u043c\u0430\u0442\u044c \u043c\u0435\u0441\u0442\u043e \u0441\u043f\u0440\u043e\u0441\u0438\u0442\u044c
\u0447\u0435\u0440\u0435\u0437 \u043b\u0438\u0446\u043e \u0447\u0442\u043e \u0442\u043e\u0433\u0434\u0430
\u0432\u0435\u0434\u044c \u0445\u043e\u0440\u043e\u0448\u0438\u0439 \u043a\u0430\u0436\u0434\u044b\u0439
\u043d\u043e\u0432\u044b\u0439 \u0436\u0438\u0442\u044c \u0434\u043e\u043b\u0436\u043d\u044b\u0439
\u0441\u043c\u043e\u0442\u0440\u0435\u0442\u044c \u043f\u043e\u0447\u0435\u043c\u0443
\u043f\u043e\u0442\u043e\u043c\u0443 \u0441\u0442\u043e\u0440\u043e\u043d\u0430 \u043f\u0440\u043e\u0441\u0442\u043e
\u043d\u043e\u0433\u0430 \u0441\u0438\u0434\u0435\u0442\u044c \u043f\u043e\u043d\u044f\u0442\u044c
\u0438\u043c\u0435\u0442\u044c \u043a\u043e\u043d\u0435\u0447\u043d\u044b\u0439 \u0434\u0435\u043b\u0430\u0442\u044c
\u0432\u0434\u0440\u0443\u0433 \u043d\u0430\u0434 \u0432\u0437\u044f\u0442\u044c \u043d\u0438\u043a\u0442\u043e
\u0441\u0434\u0435\u043b\u0430\u0442\u044c \u0434\u0432\u0435\u0440\u044c \u043f\u0435\u0440\u0435\u0434
\u043d\u0443\u0436\u043d\u044b\u0439 \u043f\u043e\u043d\u0438\u043c\u0430\u0442\u044c
\u043a\u0430\u0437\u0430\u0442\u044c\u0441\u044f \u0440\u0430\u0431\u043e\u0442\u0430 \u0442\u0440\u0438
\u0432\u0430\u0448 \u0443\u0436 \u0437\u0435\u043c\u043b\u044f \u043a\u043e\u043d\u0435\u0446
\u043d\u0435\u0441\u043a\u043e\u043b\u044c\u043a\u043e \u0447\u0430\u0441 \u0433\u043e\u043b\u043e\u0441
\u0433\u043e\u0440\u043e\u0434 \u043f\u043e\u0441\u043b\u0435\u0434\u043d\u0438\u0439 \u043f\u043e\u043a\u0430
\u0445\u043e\u0440\u043e\u0448\u043e \u0434\u0430\u0432\u0430\u0442\u044c \u0432\u043e\u0434\u0430
\u0431\u043e\u043b\u0435\u0435 \u0445\u043e\u0442\u044f \u0432\u0441\u0435\u0433\u0434\u0430
\u0432\u0442\u043e\u0440\u043e\u0439 \u043a\u0443\u0434\u0430 \u043f\u043e\u0439\u0442\u0438
\u0441\u0442\u043e\u043b \u0440\u0435\u0431\u0451\u043d\u043e\u043a \u0443\u0432\u0438\u0434\u0435\u0442\u044c
\u0441\u0438\u043b\u0430 \u043e\u0442\u0435\u0446 \u0436\u0435\u043d\u0449\u0438\u043d\u0430
\u043c\u0430\u0448\u0438\u043d\u0430 \u0441\u043b\u0443\u0447\u0430\u0439 \u043d\u043e\u0447\u044c
\u0441\u0440\u0430\u0437\u0443 \u043c\u0438\u0440 \u0441\u043e\u0432\u0441\u0435\u043c
\u043e\u0441\u0442\u0430\u0442\u044c\u0441\u044f \u043e\u0431 \u0432\u0438\u0434 \u0432\u044b\u0439\u0442\u0438
\u0434\u0430\u0442\u044c \u0440\u0430\u0431\u043e\u0442\u0430\u0442\u044c \u043b\u044e\u0431\u0438\u0442\u044c
\u0441\u0442\u0430\u0440\u044b\u0439 \u043f\u043e\u0447\u0442\u0438 \u0440\u044f\u0434
\u043e\u043a\u0430\u0437\u0430\u0442\u044c\u0441\u044f \u043d\u0430\u0447\u0430\u043b\u043e
\u0442\u0432\u043e\u0439 \u0432\u043e\u043f\u0440\u043e\u0441 \u043c\u043d\u043e\u0433\u043e
\u0432\u043e\u0439\u043d\u0430 \u0441\u043d\u043e\u0432\u0430 \u043e\u0442\u0432\u0435\u0442\u0438\u0442\u044c
\u043c\u0435\u0436\u0434\u0443 \u043f\u043e\u0434\u0443\u043c\u0430\u0442\u044c \u043e\u043f\u044f\u0442\u044c
\u0431\u0435\u043b\u044b\u0439 \u0434\u0435\u043d\u044c\u0433\u0438 \u0437\u043d\u0430\u0447\u0438\u0442\u044c
\u043f\u0440\u043e \u043b\u0438\u0448\u044c \u043c\u0438\u043d\u0443\u0442\u0430 \u0436\u0435\u043d\u0430
'''

# only add 4-byte unicode if 4-byte unicode is supported
if sys.maxunicode > 65535:
    unicode_paragraphs += '''
\U0002070e \U00020731 \U00020779 \U00020c53 \U00020c78 \U00020c96 \U00020ccf \U00020cd5 \U00020d15 \U00020d7c
\U00020d7f \U00020e0e \U00020e0f \U00020e77 \U00020e9d \U00020ea2 \U00020ed7 \U00020ef9 \U00020efa \U00020f2d
\U00020f2e \U00020f4c \U00020fb4 \U00020fbc \U00020fea \U0002105c \U0002106f \U00021075 \U00021076 \U0002107b
\U000210c1 \U000210c9 \U000211d9 \U000220c7 \U000227b5 \U00022ad5 \U00022b43 \U00022bca \U00022c51 \U00022c55
\U00022cc2 \U00022d08 \U00022d4c \U00022d67 \U00022eb3 \U00023cb7 \U000244d3 \U00024db8 \U00024dea \U0002512b
\U00026258 \U000267cc \U000269f2 \U000269fa \U00027a3e \U0002815d \U00028207 \U000282e2 \U00028cca \U00028ccd
\U00028cd2 \U00029d98
'''

ascii_words = re.split(r'\s+', ascii_paragraphs.strip())
unicode_words = re.split(r'\s+', unicode_paragraphs.strip())
words_str = ascii_words + unicode_words

mobile = [134, 135, 136, 137, 138, 139, 147, 150, 151, 152, 157, 158, 159, 172, 178, 182, 183, 184, 187, 188, 195, 197,
          198]
unicom = [130, 131, 132, 145, 155, 156, 166, 175, 176, 185, 186, 196]
telecom = [133, 149, 153, 180, 181, 189, 173, 177, 190, 191, 193, 199]


import os
import re
import time
import subprocess

# import allure
import allure
import requests
import six
import uiautomator2 as u2

from kuto.core.android.common import check_device
from kuto.utils.exceptions import (
    ScreenFailException,
    PkgIsNull,
    DeviceNotFoundException)
from kuto.utils.log import logger
from kuto.testdata import get_int


class AndroidDriver(object):

    def __init__(self, device_id=None, pkg_name=None):
        self.pkg_name = pkg_name
        if not self.pkg_name:
            raise PkgIsNull('应用包名不能为空')
        self.device_id = device_id
        if not self.device_id:
            raise DeviceNotFoundException('设备id不能为空')

        logger.info(f'init android driver: {device_id}')
        if ':' not in device_id:
            self.device_id = check_device(device_id)
            self.d = u2.connect(self.device_id)
        else:
            self.d = u2.connect_adb_wifi(self.device_id)

        # check if atx is ready.
        self.d.healthcheck()

    @property
    def info(self):
        """连接信息"""
        logger.info(f"get info")
        return self.d.info

    @property
    def app_info(self, pkg_name=None):
        """获取指定应用信息"""
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f"get app info: {pkg_name}")
        info = self.d.app_info(pkg_name)
        return info

    @property
    def device_info(self):
        """获取设备信息"""
        logger.info(f"get device info")
        info = self.d.device_info
        return info

    @property
    def page_content(self):
        """获取页面xml内容"""
        logger.info("get page content")
        return self.d.dump_hierarchy()

    def set_delay(self, before: float, after: float):
        """设置操作前置后置默认延时"""
        self.d.settings['operation_delay'] = (before, after)

    def uninstall_app(self, pkg_name=None):
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f"uninstall: {pkg_name}")
        self.d.app_uninstall(pkg_name)

    @staticmethod
    def download_apk(src):
        """下载安装包"""
        if isinstance(src, six.string_types):
            if re.match(r"^https?://", src):
                logger.info(f'下载中: {src}')
                file_path = os.path.join(os.getcwd(), src.split('/')[-1])
                r = requests.get(src, stream=True)
                if r.status_code != 200:
                    raise IOError(
                        "Request URL {!r} status_code {}".format(src, r.status_code))
                with open(file_path, 'wb') as f:
                    f.write(r.content)
                logger.info(f'下载成功: {file_path}')
                return file_path
            elif os.path.isfile(src):
                return src
            else:
                raise IOError("file {!r} not found".format(src))

    def install_app(self, apk_path, auth=True, new=True, pkg_name=None):
        """
        安装应用，push改成adb命令之后暂时无法支持远程手机调用
        @param pkg_name: 应用包名
        @param apk_path: 安装包链接，支持本地路径以及http路径
        @param auth: 是否进行授权
        @param new: 是否先卸载再安装
        """
        logger.info(f"install: {apk_path}")
        # 卸载
        if new is True:
            if pkg_name is None:
                pkg_name = self.pkg_name
            self.uninstall_app(pkg_name)

        # 下载
        source = self.download_apk(apk_path)

        # 把安装包push到手机上
        target = "/data/local/tmp/_tmp.apk"
        subprocess.check_call(f'adb -s {self.device_id} push {source} {target}', shell=True)

        # 安装
        cmd_list = ['pm', 'install', "-r", "-t", target]
        if auth is True:
            cmd_list.insert(4, '-g')
        logger.debug(f"{' '.join(cmd_list)}")
        cmd_str = f'adb -s {self.device_id} shell {" ".join(cmd_list)}'
        subprocess.check_call(cmd_str, shell=True)

        logger.info('install success')

        # 删除下载的安装包
        if 'http' in apk_path:
            os.remove(source)

    def start_app(self, pkg_name=None, stop=True):
        """启动应用
        @param pkg_name: 应用包名
        @param stop: 是否先关闭应用再启动
        """
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f"start app: {pkg_name}")
        self.d.app_start(pkg_name, stop=stop, use_monkey=True)

    def stop_app(self, pkg_name=None):
        """停止指定应用"""
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f"quit app: {pkg_name}")
        self.d.app_stop(pkg_name)

    def clear_app(self, pkg_name=None):
        """清除应用缓存"""
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f"clear app: {pkg_name}")
        self.d.app_clear(pkg_name)

    def wait_app_running(self, pkg_name=None, front=True, timeout=20):
        """
        等待应用运行
        @param pkg_name: 应用包名
        @param front: 是否前台运行
        @param timeout: 等待时间
        @return: 应用pid
        """
        if not pkg_name:
            pkg_name = self.pkg_name
        pid = self.d.app_wait(pkg_name, front=front, timeout=timeout)
        if not pid:
            logger.info(f"{pkg_name} is not running")
        else:
            logger.info(f"{pkg_name} pid is {pid}")
        return pid

    def health_check(self):
        """检查设备连接状态"""
        logger.info("health check")
        self.d.healthcheck()

    def open_url(self, url):
        """
        通过url打开web页面或者app schema
        @param url: 页面url，https://www.baidu.com，taobao://taobao.com
        @return:
        """
        logger.info(f"open url or schema: {url}")
        self.d.open_url(url)

    def shell(self, cmd, timeout=60):
        """
        执行adb shell命令
        @param cmd: shell字符串或list，pwd，["ls", "-l"]
        @param timeout: 超时时间
        @return:
        """
        logger.info(f"exec adb shell: {cmd}")
        output, exit_code = self.d.shell(cmd, timeout=timeout)
        return output, exit_code

    def screenshot(self, file_name=None, with_time=True):
        """
        截图并保存到预定路径
        @param with_time: 是否带时间戳
        @param file_name: foo.png or fool
        @return:
        """
        if not file_name:
            raise ValueError("file_name should not be None.")

        try:
            # 把文件名处理成test.png的样式
            if "." in file_name:
                file_name = file_name.split(r".")[0]
            # 截图并保存到当前目录的images文件夹中
            relative_path = "screenshots"
            if os.path.exists(relative_path) is False:
                os.mkdir(relative_path)
            if with_time:
                time_str = time.strftime(f"%Y-%m-%d_%H_%M_%S_{get_int(min_size=1, max_size=1000)}")
                file_name = f"{time_str}_{file_name}.png"
            else:
                file_name = f'{file_name}.png'

            file_path = os.path.join(relative_path, file_name)
            logger.info(f"save to: {os.path.join(relative_path, file_name)}")
            self.d.screenshot(file_path)
            # 上传allure报告
            allure.attach.file(
                file_path,
                attachment_type=allure.attachment_type.PNG,
                name=f"{file_name}",
            )
            return file_path
        except Exception as e:
            raise ScreenFailException(f"screenshot fail: \n{str(e)}")

    def back(self):
        logger.info(f'back')
        self.d.press('back')

    def click(self, x, y):
        """点击坐标"""
        logger.info(f"click : {x},{y}")
        self.d.click(x, y)

    def click_alerts(self, alert_list: list):
        """点击弹窗"""
        logger.info(f"click alerts: {alert_list}")
        with self.d.watch_context() as ctx:
            for alert in alert_list:
                ctx.when(alert).click()
            ctx.wait_stable()

    def swipe(self, sx, sy, ex, ey):
        """滑动"""
        logger.info(f"from {sx},{sy} swipe to {ex},{ey}")
        self.d.swipe(sx, sy, ex, ey)

    def swipe_left(self, scale=0.9):
        """往左滑动"""
        logger.info("swipe left")
        self.d.swipe_ext("left", scale=scale)

    def swipe_right(self, scale=0.9):
        """往右滑动"""
        logger.info("swipe right")
        self.d.swipe_ext("right", scale=scale)

    def swipe_up(self, scale=0.8):
        """往上滑动"""
        logger.info("swipe up")
        self.d.swipe_ext("up", scale=scale)

    def swipe_down(self, scale=0.8):
        """往下滑动"""
        logger.info("swipe down")
        self.d.swipe_ext("down", scale=scale)

    def drag(self, sx, sy, ex, ey):
        """拖动"""
        logger.info(f"from {sx},{sy} drag to {ex},{ey}")
        self.d.drag(sx, sy, ex, ey)

    def set_text(self, text: str):
        """输入框内容"""
        logger.info(f'input: {text}')
        self.d.send_keys(text)


if __name__ == '__main__':
    pass






# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages
from kuto import __version__, __description__

try:
    long_description = open(os.path.join('kuto', "README.md"), encoding='utf-8').read()
except IOError:
    long_description = ""

setup(
    name="kuto",
    version=__version__,
    description=__description__,
    author="杨康",
    author_email="772840356@qq.com",
    url="https://gitee.com/bluepang2021/kuto",
    platforms="Android,IOS,Web,Api",
    packages=find_packages(),
    long_description=long_description,
    python_requires='>=3.9',
    classifiers=[
        "Programming Language :: Python :: 3.9"
    ],
    include_package_data=True,
    package_data={
        r'': ['*.yml'],
    },
    install_requires=[
        'requests-toolbelt==0.10.1',
        'jmespath==0.9.5',
        'jsonschema==4.17.0',
        'uiautomator2==2.16.13',
        'tidevice==0.6.1',
        'facebook-wda==1.4.6',
        'playwright==1.33.0',
        'pytest==6.2.5',
        'pytest-rerunfailures==10.2',
        'pytest-ordering==0.6',
        'pytest-xdist==2.5.0',
        'pytest-dependency==0.5.1',
        'allure-pytest==2.9.45',
        'PyYAML==6.0',
        'pandas==1.3.4',
        'openpyxl==3.0.9',
        'XlsxWriter==3.0.2',
        'yagmail==0.15.293',
        'PyMySQL==0.10.1',
        'pymongo==4.0.1',
        'click~=8.1.3'
    ],
    extras_require={
        "ocr": ["easyocr==1.6.2"],
        "opencv": [
            'opencv-python==4.6.0.66',
            'opencv-contrib-python==4.6.0.66',
            'opencv-python-headless==3.4.18.65'
        ]
    },
    entry_points={
        'console_scripts': [
            'kuto = kuto.cli:main'
        ]
    },
)



import sys


def general_api_case(module: str, host: str, case_name: str, method: str, url: str, assert_data: dict, headers: dict = None,
                     payload: dict = None):
    """生成api用例
    @param host:
    @param module: 所属模块
    @param case_name：用例名称
    @param method: 请求方法
    @param url: 请求url
    @param headers: 请求头
    @param payload: 请求数据
    @param assert_data: 断言数据
    {
        "path": "data.id",
        "method": "大于",
        "expect": 0
    }
    """
    api_name = url.split('/')[-1]  # 接口路径最后一段
    class_name = 'Test' + api_name.title()

    if method == 'get':
        payload_str = ', data=payload'
    elif method == 'post':
        payload_str = ', json=payload'
    else:
        payload_str = ''

    assert_method = assert_data.get('method', None)
    assert_path = assert_data.get('path', None)
    if assert_method == '等于':
        assert_str = f"self.assert_eq('{assert_path}')"
    elif assert_data == '大于':
        assert_str = f"self.assert_gt('{assert_path}')"
    else:
        sys.exit('暂不支持的断言方式')

    if headers:
        headers_str = f', headers={headers}'
    else:
        headers_str = ''

    api_case_template = f'''import qrunner
from qrunner import *


@module("{module}")
class {class_name}(qrunner.TestCase):

    @title('{case_name}')
    def test_{api_name}(self):
        payload = {payload}
        self.{method}({url}{payload_str}{headers_str})
        {assert_str}


if __name__ == '__main__':
    qrunner.main(
        host='{host}'
    )
'''

    return api_case_template


if __name__ == '__main__':
    print(general_api_case(
        module='查专利',
        host='https://patents-test.qizhidao.com',
        case_name='简单检索',
        method='get',
        url='/patentSearch/search',
        payload={"key": "华为"},
        headers={"token": "xxx"},
        assert_data={
            "path": "code",
            "method": "等于",
            "expect": 0
        }
    ))



import traceback
import yagmail
from kuto.utils.allure_data import get_allure_data
from kuto.utils.log import logger
from kuto.testdata import get_now_datetime


class Mail:
    def __init__(self, host, port, user, password):
        self.host = host
        self.port = port
        self.username = user
        self.password = password

    def send_email(self, receivers, subject, contents, attachments=None):
        """
        发送邮件
        @param receivers: 收件人列表
        @param subject: 邮件主题
        @param contents: 邮件内容
        @param attachments: 附件文件的绝对路径
        @return:
        """
        try:
            logger.info(f'开始发送邮件~')
            # 初始化服务对象直接根据参数给定，更多参考SMTP(）内部
            server = yagmail.SMTP(host=self.host, port=self.port,
                                  user=self.username, password=self.password)
            # 发送内容，设置接受人等信息，更多参考SMTP.send()内部
            server.send(to=receivers,
                        subject=subject,
                        contents=contents,
                        attachments=attachments)
            server.close()
        except Exception:
            logger.info(f'发送失败~')
            print('traceback.format_exc(): {}'.format(traceback.format_exc()))
            return False

        # 无任何异常表示发送成功
        logger.info(f'发送成功~')
        return True

    # def send_mail(self, mail_data, receivers):
    #     """
    #     参考：https://blog.csdn.net/fenglepeng/article/details/107005000
    #     @param mail_data:
    #     @param receivers:
    #     @return:
    #     """
    #     print(f'向{receivers}发送邮件...')
    #     # 创建一个带附件的实例
    #     message = MIMEMultipart()
    #     message['From'] = Header(self.username)
    #     message['To'] = Header(",".join(receivers))
    #     message['Subject'] = Header(mail_data.get('title'), 'utf-8')
    #
    #     # 邮件正文内容
    #     message.attach(MIMEText(mail_data.get('body'), 'plain', 'utf-8'))
    #     # 附件
    #     file_path = mail_data.get('file_path')
    #     if file_path:
    #         # 构造附件，传送当前目录下的文件
    #         att1 = MIMEText(open(file_path, 'r').read(), 'base64', 'utf-8')
    #         att1["Content-Type"] = 'application/octet-stream'
    #         # 这里的filename可以任意写，写什么名字，邮件中显示什么名字
    #         file_name = mail_data.get('file_name')
    #         att1["Content-Disposition"] = f'attachment; filename="{file_name}"'
    #         message.attach(att1)
    #
    #     # 连接
    #     conn = smtplib.SMTP_SSL(self.host, 465)
    #     # 登录
    #     conn.login(self.username, self.password)
    #     # 发送邮件
    #     try:
    #         conn.sendmail(self.username, receivers, message.as_string())
    #     except Exception as e:
    #         print(f'发送失败: {str(e)}')
    #     else:
    #         print('发送成功')
    #     # 断开连接
    #     conn.quit()

    def send_report(self, to_list, title, reports_url):
        allure_data = get_allure_data('reports')
        total = allure_data.get('total')
        fail = allure_data.get('failed')
        passed = allure_data.get('passed')
        rate = allure_data.get('rate')

        # 邮件内容
        title = f'{title}({get_now_datetime(strftime=True)})'
        body_str = '\n\n共 {0} 个用例，通过 {1} 个，失败 {2} 个，通过率 {3}%，详见: {4}'\
            .format(total, passed, fail, rate, reports_url)

        # 发送邮件
        if fail > 0:
            self.send_email(to_list, title, body_str)
        else:
            logger.info('用例全部成功，无需通知~')


if __name__ == '__main__':
    pass








import os
import time

from kuto.core.ios.driver import IosDriver
from kuto.utils.log import logger
from kuto.core.image.image_discern import ImageDiscern
from kuto.utils.exceptions import ElementNameEmptyException


class ImageElem(object):
    """图像识别定位"""

    def __init__(self, driver=None, image: str = None, desc: str = None):
        self.driver = driver
        self.target_image = image
        if desc is None:
            raise ElementNameEmptyException("请设置控件名称")
        else:
            self._desc = desc

    def __get__(self, instance, owner):
        if instance is None:
            return None

        self.driver = instance.driver
        return self

    def exists(self, retry=3, timeout=3, grade=0.9, gauss_num=111):
        logger.info(f'图像识别判断: {self.target_image} 是否存在')
        time.sleep(3)
        for i in range(retry):
            logger.info(f'第{i + 1}次查找:')
            self.driver.screenshot('SourceImage.png', with_time=False)
            res = ImageDiscern(self.target_image, grade, gauss_num).get_coordinate()
            logger.debug(res)
            if isinstance(res, tuple):
                self.driver.screenshot(f'图像识别定位-{self.target_image}')
                return True
            time.sleep(timeout)
        else:
            self.driver.screenshot(f'图像识别定位失败-{self.target_image}')
            return False

    def click(self, retry=3, timeout=3, grade=0.9, gauss_num=111):
        logger.info(f'图像识别点击图片: {self.target_image}')
        time.sleep(3)
        for i in range(retry):
            logger.info(f'第{i + 1}次查找:')
            self.driver.screenshot('SourceImage.png', with_time=False)
            res = ImageDiscern(self.target_image, grade, gauss_num).get_coordinate()
            if isinstance(res, tuple):
                logger.info(f'识别坐标为: {res}')
                self.driver.screenshot(f'图像识别定位-{self.target_image}')
                x, y = res[0], res[1]
                if isinstance(self.driver, IosDriver):
                    x, y = int(x/self.driver.d.scale), int(y/self.driver.d.scale)
                self.driver.click(x, y)
                return
            time.sleep(timeout)
        else:
            self.driver.screenshot(f'图像识别定位失败-{self.target_image}')
            raise Exception('未识别到图片，无法进行点击')


if __name__ == '__main__':
    from kuto.core.android.driver import AndroidDriver

    driver = AndroidDriver()
    driver.pkg_name = 'com.qizhidao.clientapp'
    driver.start_app()
    driver.d(resourceId='com.qizhidao.clientapp:id/bottom_btn').click(timeout=5)
    # elem = OCRElem(driver, '查老板')
    # elem.click()

    elem = ImageElem(driver, 'tpl1670743672123.png')
    elem.click(grade=0.9)



import kuto


if __name__ == '__main__':
    # 主程序入口

    # 执行接口用例
    # kuto.main(
    #     host='https://app-pre.qizhidao.com',
    #     path='tests/test_api.py'
    # )

    # 执行安卓用例
    # kuto.main(
    #     platform='android',
    #     device_id='UJK0220521066836',
    #     pkg_name='com.qizhidao.clientapp',
    #     path='tests/test_adr.py'
    # )

    # 执行ocr用例
    # kuto.main(
    #     platform='android',
    #     device_id='UJK0220521066836',
    #     pkg_name='com.qizhidao.clientapp',
    #     path='tests/test_ocr.py'
    # )

    # 执行图像识别用例
    # kuto.main(
    #     platform='ios',
    #     device_id='00008101-000E646A3C29003A',
    #     pkg_name='com.qizhidao.company',
    #     path='tests/test_image.py'
    # )

    # 执行ios用例
    # kuto.main(
    #     platform='ios',
    #     device_id='00008101-000E646A3C29003A',
    #     pkg_name='com.qizhidao.company',
    #     path='tests/test_ios.py'
    # )

    # 执行web用例
    kuto.main(
        platform='web',
        host='https://www.qizhidao.com/',
        path='tests/web',
        xdist=False
    )
